"""
Tooling for the ERC-7730 standard.
"""

# Base directory for descriptors in https://github.com/LedgerHQ/clear-signing-erc7730-registry.
ERC_7730_REGISTRY_DIRECTORY = "registry"

# File name prefix for EIP-712 descriptors in https://github.com/LedgerHQ/clear-signing-erc7730-registry.
ERC_7730_REGISTRY_EIP712_PREFIX = "eip712-"

# File name prefix for calldata descriptors in https://github.com/LedgerHQ/clear-signing-erc7730-registry.
ERC_7730_REGISTRY_CALLDATA_PREFIX = "calldata-"
