import { UI } from "~/ui/UI";

export const SelectValues = () => {
  return (
    <div>
      <UI.Label>Preview with</UI.Label>
      <UI.FauxInput>Placeholder values</UI.FauxInput>
    </div>
  );
};
